/**
 * Copyright (c) 2004 Logos GmbH Rostock (www.logos-ib.de).
 * All rights reserved.
 *
 * THIS SOFTWARE IS  PROVIDED  "AS IS"  AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF  MERCHANTABILITY  AND  FITNESS  FOR  A PARTICULAR  PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL  LOGOS GMBH  OR ITS CONTRIBUTORS BE
 * LIABLE FOR ANY  DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY,
 * OR CONSEQUENTIAL DAMAGES  (INCL., BUT NOT LIMITED TO, PROCUREMENT
 * OF SUBSTITUTE GOODS OR SERVICES;  LOSS OF USE, DATA, OR  PROFITS;
 * OR BUSINESS INTERRUPTION)  HOWEVER  CAUSED  AND  ON ANY THEORY OF
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,  OR TORT (INCL.
 * NEGLIGENCE OR OTHERWISE)  ARISING  IN  ANY  WAY OUT OF THE USE OF
 * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */
package de.bsvrz.dua.bastband.bastband.dvs.funclib.objectwrapper;

/**
 * History:
 *
 * $Log: EAK.java,v $
 * Revision 1.2  2007/07/25 13:02:07  Trier
 * <Kein Kommentar eingegeben>
 *
 * Revision 1.1  2007/07/04 11:43:42  Trier
 * Neuordnung
 *
 * Revision 1.1.1.1  2007/06/20 09:56:26  Trier
 * dvs alles
 *
 * Revision 1.6  2006/04/28 05:48:49  Trier
 * Generics 1.5 verwendet
 *
 * Revision 1.5  2005/11/09 14:23:04  Trier
 * <Kein Kommentar eingegeben>
 *
 * Revision 1.4  2005/09/28 08:02:37  Trier
 * <Kein Kommentar eingegeben>
 *
 * Revision 1.3  2005/08/17 09:43:21  Trier
 * <Kein Kommentar eingegeben>
 *
 * Revision 1.2  2005/01/25 09:43:11  Trier
 * <Kein Kommentar eingegeben>
 *
 * Revision 1.1  2005/01/21 13:25:56  Trier
 * <Kein Kommentar eingegeben>
 *
 */

import java.util.ArrayList;
import java.util.List;
import java.util.WeakHashMap;

import de.bsvrz.dav.daf.main.Data;
import de.bsvrz.dav.daf.main.config.AttributeGroup;
import de.bsvrz.dav.daf.main.config.ConfigurationObject;
import de.bsvrz.dav.daf.main.config.ConfigurationObjectType;
import de.bsvrz.dav.daf.main.config.DataModel;
import de.bsvrz.dav.daf.main.config.ObjectSet;
import de.bsvrz.dav.daf.main.config.SystemObject;
import de.bsvrz.dua.bastband.bastband.dvs.funclib.DataModellMismatchException;
import de.bsvrz.dua.bastband.bastband.dvs.funclib.DataModellObjectNotFoundException;
import de.bsvrz.dua.bastband.bastband.dvs.funclib.LogosClientDavConnection;

/**
 * kapselt den DatenModellTyp eak
 */
public class EAK extends ConfigurationObjectWrapper {

	private static WeakHashMap<String, EAK> __Cache = new WeakHashMap<String, EAK>();

	private static ConfigurationObjectType __ConfigurationObjectType;
	private static ConfigurationObjectType __ConfigurationObjectTypeOfDE;
	private static AttributeGroup __AtgEAK;

	private String _Bezeichnung;
	private int _OSI2Adresse;
	private String _UmsetzungsModul;

	private ArrayList<DEKanal> _DEList;
	private DEKanal _DEKanal;
	private SteuerModul _SteuerModul;

	/**
	 * setzt das SteuerModul, an den der EAK angeschlossen ist.
	 *
	 * @param steuerModul
	 *            SteuerModul, an den der EAK angeschlossen ist.
	 */
	public void setSteuerModul(final SteuerModul steuerModul) {
		_SteuerModul = steuerModul;
	}

	/**
	 * gibt das SteuerModul zur&uuml;ck, an den der SteuerModul angeschlossen
	 * ist, und dieses gesetzt wurde.
	 *
	 * @return SteuerModul, an den der EAK angeschlossen ist.
	 */
	public SteuerModul getSteuerModul() {
		return _SteuerModul;
	}

	/**
	 * gibt die Bezeichnung zur&uuml;ck.
	 *
	 * @return String
	 */
	public String getBezeichnung() {
		return _Bezeichnung;
	}

	/**
	 * gibt die EAK-Port-Nummer auf dem Lokalbus zur&uuml;ck.
	 *
	 * @return die OSI2Adresse
	 */
	public int getOSI2Adresse() {
		return _OSI2Adresse;
	}

	/**
	 * gibt das für die Umsetzung des Protokolls (Lesen und Schreiben)
	 * verwendete Modul an diesem Knoten zur&uuml;ck.
	 *
	 * @return Klassennamen des UmsetzungsModuls.
	 */
	public String getUmsetzungsModul() {
		return _UmsetzungsModul;
	}

	/**
	 * gibt die DE-Kan&auml;le zur&uuml;ck oder null, wenn keine angeschlossen
	 * sind.
	 *
	 * @return DE-Kan&auml;le.
	 */
	public DEKanal[] getDEKanals() {
		if ((_DEList != null) && (_DEList.size() > 0)) {
			return _DEList.toArray(new DEKanal[_DEList.size()]);
		} else {
			return null;
		}
	}

	/**
	 * erzeugt das WrapperObject f&uuml;r das Object mit der angegebenen Pid
	 *
	 * @see ConfigurationObjectWrapper#ConfigurationObjectWrapper(String pid)
	 *
	 * @param pid
	 *            String
	 * @throws DataModellMismatchException
	 *             ungültiges Datenmodell
	 * @throws DataModellObjectNotFoundException
	 *             ungültiges Datenmodell
	 */
	protected EAK(final String pid) throws DataModellMismatchException, DataModellObjectNotFoundException {
		super(pid);
	}

	/**
	 * erzeugt das WrapperObject f&uuml;r das angegebene Object.
	 *
	 * @see de.bsvrz.dua.bastband.bastband.dvs.funclib.objectwrapper.ConfigurationObjectWrapper#ConfigurationObjectWrapper(ConfigurationObject
	 *      confObj)
	 *
	 * @param confObj
	 *            zu wrappenes ConfigurationObject, das ein EAK
	 *            repr&auml;sentieren muss.
	 * @throws DataModellMismatchException
	 *             ungültiges Datenmodell
	 */
	protected EAK(final ConfigurationObject confObj) throws DataModellMismatchException {
		super(confObj);
	}

	@Override
	protected void initData(final ConfigurationObject confObj) throws DataModellMismatchException {

		// if (__AtgEAK == null) {
		// DataModel model =
		// LogosClientDavConnection.getInstance().getDataModel();
		// __AtgEAK = model.getAttributeGroup("atg.eak");
		// __ConfigurationObjectTypeOfDE =
		// (ConfigurationObjectType)model.getType("typ.de");
		// }
		final Data oData = confObj.getConfigurationData(EAK.__AtgEAK);
		if (oData != null) {
			_Bezeichnung = oData.getTextValue("Bezeichnung").getText();
			_OSI2Adresse = oData.getUnscaledValue("OSI2Adresse").intValue();
			_UmsetzungsModul = oData.getTextValue("UmsetzungsModul").getText();
		}
		final ObjectSet os = confObj.getObjectSet("De");
		if (os != null) {
			final List list = os.getElements();
			if ((list != null) && (list.size() > 0)) {
				_DEList = new ArrayList<DEKanal>(list.size());
				for (int i = 0; i < list.size(); i++) {
					final ConfigurationObject deObj = (ConfigurationObject) list.get(i);
					if (deObj.isOfType(EAK.__ConfigurationObjectTypeOfDE)) {
						final DEKanal deKanal = DEKanal.getInstance(deObj);
						deKanal.setEAK(this);
						_DEList.add(deKanal);
					}
				}
			}
		}
		EAK.__Cache.put(this.getPID(), this);
	}

	@Override
	ConfigurationObjectType getType() {
		if (EAK.__ConfigurationObjectType == null) {
			final DataModel model = LogosClientDavConnection.getInstance().getDataModel();
			EAK.__ConfigurationObjectType = (ConfigurationObjectType) model.getType("typ.eak");

			EAK.__AtgEAK = model.getAttributeGroup("atg.eak");
			EAK.__ConfigurationObjectTypeOfDE = (ConfigurationObjectType) model.getType("typ.de");

			final List<SystemObject> list = EAK.__ConfigurationObjectType.getObjects();
			model.getConfigurationData(list, EAK.__AtgEAK);
		}
		return EAK.__ConfigurationObjectType;
	}

	/**
	 * gibt den DEKanal zur Steurung des EAK's zur&uuml;ck.
	 *
	 * @return DEKanal oder null, wenn der EAK ein virtueller ist (EAK254
	 *         OsiAddr=0).
	 */
	public DEKanal getSteuerDEKanal() {
		return _DEKanal;
	}

	/**
	 * setzt den DEKanal zur Steurung des EAK's (nur, wenn der EAK nicht
	 * virtuelle ist).
	 *
	 * @param deKanal
	 *            DEKanal &uuml;ber den auf den EAK zugegriffen werden kann.
	 */
	public void setDEKanal(final DEKanal deKanal) {
		_DEKanal = deKanal;
	}

	public DEKanal findDEKanal(final int osiAddr) {

		for (int i = 0; i < _DEList.size(); i++) {
			final DEKanal deKanal = _DEList.get(i);
			if (deKanal.getDEKanal() == osiAddr) {
				return deKanal;
			}
		}
		return null;
	}

	@Override
	public String getTreeString() {
		final String s = getBezeichnung();
		final int index = s.indexOf("(", 2);
		if (index > 0) {
			return s.substring(0, index);
		}
		return s;
	}

}
