/**
 * Copyright (c) 2004 Logos GmbH Rostock (www.logos-ib.de).
 * All rights reserved.
 *
 * THIS SOFTWARE IS  PROVIDED  "AS IS"  AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF  MERCHANTABILITY  AND  FITNESS  FOR  A PARTICULAR  PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL  LOGOS GMBH  OR ITS CONTRIBUTORS BE
 * LIABLE FOR ANY  DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY,
 * OR CONSEQUENTIAL DAMAGES  (INCL., BUT NOT LIMITED TO, PROCUREMENT
 * OF SUBSTITUTE GOODS OR SERVICES;  LOSS OF USE, DATA, OR  PROFITS;
 * OR BUSINESS INTERRUPTION)  HOWEVER  CAUSED  AND  ON ANY THEORY OF
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,  OR TORT (INCL.
 * NEGLIGENCE OR OTHERWISE)  ARISING  IN  ANY  WAY OUT OF THE USE OF
 * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */
package de.bsvrz.dua.bastband.bastband.dvs.kex.datenaustausch.output;

import java.io.BufferedReader;
import java.io.File;
import java.io.InputStreamReader;
import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Calendar;
import java.util.Date;
import java.util.List;

import de.bsvrz.dua.bastband.bastband.dvs.funclib.BaseConsoleProgram;
import de.bsvrz.dua.bastband.bastband.dvs.funclib.objectwrapper.LangZeitZaehlStelle;
import de.bsvrz.sys.funclib.commandLineArgs.ArgumentList;

/**
 * <p>
 * Zur Verwendung siehe {@link #printVerwendung()}
 * </p>
 *
 * <p>
 * <b>Achtung:</b> vor Aufruf der run-Methode muss
 * {@link de.bsvrz.dua.bastband.bastband.dvs.funclib.LogosClientDavConnection}
 * initialisiert sein.
 *
 * @author Ralf Trier
 * @version 1.0
 */
public class BastExportApp extends BaseConsoleProgram {

	private static de.bsvrz.dua.bastband.bastband.funclib.logger.Logger logger;

	private static SimpleDateFormat DATE_FORMAT = new SimpleDateFormat(
			"dd.MM.yyyy HH:mm");

	Calendar _CalStart;
	Calendar _CalEnd;
	File _Dir;
	int _BastFormat;
	List<String> _LzzPidList;

	/**
	 * erzeugt die Application
	 *
	 * @param dir
	 *            Verzeichnis in das die Dateien gespeichert werden sollen.
	 * @param iBastFormat
	 *            einer der Werte aus {@link BastExporter}
	 * @param von
	 *            Date
	 * @param bis
	 *            Date
	 * @param lzzPidList
	 *            List mit den Pids der auszugebenden Zählstellen, wenn null
	 *            werden alle ausgegeben.
	 */
	public BastExportApp(final File dir, final int iBastFormat, final Date von,
			final Date bis, final List<String> lzzPidList) {
		_CalStart = Calendar.getInstance();
		_CalStart.setTime(von);
		_CalEnd = Calendar.getInstance();
		_CalEnd.setTime(bis);
		_Dir = dir;
		_BastFormat = iBastFormat;
		_LzzPidList = lzzPidList;
	}

	@Override
	public void run() {
		BastExportApp.logger = de.bsvrz.dua.bastband.bastband.funclib.logger.LoggerFactory
				.getLoggerFactory().getLogger();
		try {
			final BastExporter bastExporter = new BastExporter(_BastFormat);
			if (_LzzPidList != null) {
				for (final String pid : _LzzPidList) {
					LangZeitZaehlStelle lzz = null;
					try {
						lzz = LangZeitZaehlStelle.getInstance(pid);
					} catch (final Exception e) {
					}
					if (lzz == null) {
						BastExportApp.logger
						.info("Zählstelle mit der Pid %s wurde nicht gefunden. Pid wird ignoriert",
								pid);
					} else {
						BastExportApp.logger.info("Daten der Zählstelle " + lzz
								+ " werden ausgegeben.");
						bastExporter.export(lzz, _Dir, _CalStart, _CalEnd);
					}
				}
			} else {
				for (final LangZeitZaehlStelle lzz : LangZeitZaehlStelle
						.getAll()) {
					BastExportApp.logger.info("Daten der Zählstelle " + lzz
							+ " werden ausgegeben.");
					bastExporter.export(lzz, _Dir, _CalStart, _CalEnd);
				}
			}
		} catch (final Exception ex) {
			ex.printStackTrace();
		}
	}

	public static void main(final String[] args) {
		System.setErr(System.out);

		final ArgumentList argumentList = new ArgumentList(args);

		Date dStartDate = null;
		if (!argumentList.hasArgument("-von")) {
			while (dStartDate == null) {
				try {
					System.out
					.print("Geben Sie bitte an ab wann die Daten exportiert werden sollen [TT.MM.JJJJ SS:MM]   = ");
					final String sStartDate = (new BufferedReader(
							new InputStreamReader(System.in))).readLine();
					dStartDate = BastExportApp.DATE_FORMAT.parse(sStartDate);
				} catch (final Exception ex1) {
				}
			}
		} else {
			final String sStartDate = argumentList.fetchArgument("-von")
					.asString();
			try {
				dStartDate = BastExportApp.DATE_FORMAT.parse(sStartDate);
			} catch (final ParseException ex) {
				System.out
				.println("kein gültiges Format für Parameter \"-von\" TT.MM.JJJJ SS:MM.");
				BastExportApp.printVerwendung();
				System.exit(0);
			}
		}

		Date dEndDate = null;
		if (!argumentList.hasArgument("-bis")) {
			while (dEndDate == null) {
				try {
					System.out
					.print("Geben Sie bitte an bis wann die Daten exportiert werden sollen [TT.MM.JJJJ SS:MM]  = ");
					final String sEndDate = (new BufferedReader(
							new InputStreamReader(System.in))).readLine();
					dEndDate = BastExportApp.DATE_FORMAT.parse(sEndDate);
				} catch (final Exception ex1) {
				}
			}
		} else {
			final String sEndDate = argumentList.fetchArgument("-bis")
					.asString();
			try {
				dEndDate = BastExportApp.DATE_FORMAT.parse(sEndDate);
			} catch (final ParseException ex) {
				System.out
				.println("kein gültiges Format für Parameter \"-von\" TT.MM.JJJJ SS:MM.");
				BastExportApp.printVerwendung();
				System.exit(0);
			}
		}

		File dir = null;
		try {
			dir = argumentList.fetchArgument("-dir").asDirectory();
		} catch (final Exception e) {
			System.out
			.println("Parameter \"-dir\" ist kein gültiges Verzeichnis.");
			BastExportApp.printVerwendung();
			System.exit(0);
		}

		if (!argumentList.hasArgument("-bastformat")) {
			System.out
			.println("Parameter \"-bastformat\" muss angegeben werden.");
			BastExportApp.printVerwendung();
			System.exit(0);
		}

		ArrayList<String> lzzPidList = null;
		while (argumentList.hasArgument("-lzz")) {
			if (lzzPidList == null) {
				lzzPidList = new ArrayList<String>();
			}
			lzzPidList.add(argumentList.fetchArgument("-lzz").asString());
		}

		final String sBAStFormat = argumentList.fetchArgument("-bastformat")
				.asString();
		final int iBastFormat = BastExportApp.getBASTFormatAsInt(sBAStFormat);
		if (iBastFormat == -1) {
			System.out
			.println("Parameter \"-bastformat\" muss einer der Werte sein: V2004, SA, MV, MVTest.");
			BastExportApp.printVerwendung();
			System.exit(0);
		}

		System.out.println("Daten werden eingelesen von "
				+ BastExportApp.DATE_FORMAT.format(dStartDate) + " bis "
				+ BastExportApp.DATE_FORMAT.format(dEndDate));
		(new BastExportApp(dir, iBastFormat, dStartDate, dEndDate, lzzPidList))
		.start(args, true);

	}

	static int getBASTFormatAsInt(final String sBAStFormat) {
		if ("V2004".equalsIgnoreCase(sBAStFormat)) {
			return BastExporter.BAST_FORMAT_V2004;
		}
		if ("SA".equalsIgnoreCase(sBAStFormat)) {
			return BastExporter.BAST_FORMAT_SA;
		}
		if ("MV".equalsIgnoreCase(sBAStFormat)) {
			return BastExporter.BAST_FORMAT_MV;
		}
		if ("MVTest".equalsIgnoreCase(sBAStFormat)) {
			return BastExporter.BAST_FORMAT_MV_TEST;
		}
		return -1;
	}

	/**
	 * Verwendung wird ausgegeben.
	 *
	 * <pre>
	 * neben den Parametern für den Datenverteiler sind anzugeben:
	 *       -von="TT.MM.JJJJ SS:MM\" (wenn nicht angeben, erfolgt Abfrage)
	 *       -bis="TT.MM.JJJJ SS:MM\" (wenn nicht angeben, erfolgt Abfrage)
	 *       -bastformat=V2004|SA|MV|MVTest
	 *       -dir=directoryZumSpeichern
	 *       -lzz=lzzPid (mehrfache Angabe möglich, wird der Parameter nicht angegeben, werden die Daten aller Zählstellen ausgegeben).
	 * </pre>
	 */
	public static void printVerwendung() {
		System.out
		.println("neben den Parametern für den Datenverteiler sind anzugeben: ");
		System.out
		.println("    -von=\"TT.MM.JJJJ SS:MM\" (wenn nicht angeben, erfolgt Abfrage)");
		System.out
		.println("    -bis=\"TT.MM.JJJJ SS:MM\" (wenn nicht angeben, erfolgt Abfrage)");
		System.out.println("    -bastformat=V2004|SA|MV|MVTest");
		System.out.println("    -dir=directoryZumSpeichern");
		System.out
		.println("    -lzz=lzzPid (mehrfache Angabe möglich, wird der Parameter nicht angegeben, werden die Daten aller Zählstellen ausgegeben.");
	}

}
